///// [ LICENSE ] //////////////////////////////////////////////////////////////
//
// (c) 2009 Dynamite Data / Tom Corelis
// 
// Unless explicitly stated otherwise, the programming code in this file is
// licensed to you pursuant the terms described in codelicense.txt.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// DDFIREFOX GLOBALS JAVASCRIPT CODE MODULE
//
// Data and code in this module run as a shared instance between all scripts
// that import it. Only data and code that needs to be globally shared should be
// stored here.
//
// Note that the contents of this JSM accessible from
// com.dynamitedata.ddfGlobals outside this code module.
////////////////////////////////////////////////////////////////////////////////

// the following is requred for this module to work correctly
// every globally-accessible var/object/const needs to be in this array

var EXPORTED_SYMBOLS = [
    /* CONSTANTS */
    "SERVER_DEBUG",
    "SERVER_RELEASE",
    
    "MERCHANT_LIST_PREFIX",
    "MERCHANT_LIST_POSTFIX",
    "MERCHANT_IMAGE_URL",
    
    "UPDATE_MESSAGE_URL",
    
    "PRODUCT_INFO_PROTOCOL",
    "PRODUCT_INFO_ARGSPREFIX",
    "PRODUCT_INFO_ARGSPOSTFIX",
    "PRODUCT_INFO_MAXRESULTS",
    "PRODUCT_INFO_LISTPOSTFIX",
    "PRODUCT_INFO_REFRESH_TIMEOUT", 
    
    "SETI_PREFIX",
    "SETI_LIST_POSTFIX",
    "SETI_SUBMISSION_POSTFIX",
    
    "STATE_MANAGER_TIMEOUT_STATUS",
    "STATE_MANAGER_TIMEOUT_TABS",
    "STATE_MANAGER_TIMEOUT_ERRORS",
    "STATE_MANAGER_SCAN_INTERVAL",
    
    "MESSAGE_HIDE_DELAY_MSECS",

    "STATUS_NBR_MIN",
    "STATUS_NBR_MAX",
    
    "EULA_VERSION_LATEST",
    "DDFIREFOX_EXT_ID",
    
    "LIST_COUNTRIES_OVERRIDES",
    "LIST_COUNTRIES_NOEXPANSION",
    
    /* GLOBAL VARIABLES */
    /* note: primitive type variables need to be stored in the globals object */
    "urlTable",
    "stateTable",
    "globals",
    "failoverChain",
    
    /* INTERFACES */
    "iStrSet",
    
    /* FUNCTIONS */
    "globalStartup",
    "stateStruct",
    "stateManager",
    "getDebugLevel",
    "setDebugLevel",
    "regexFindDomain",
    "dprint",
    "ddSetStringPref",
    "ddSetBoolPref",
    "ddSetIntPref",
    "ddGetStringPref",
    "ddGetBoolPref",
    "ddGetIntPref",
    "isAlive",
    "isNumeric",
    "CloneObject",
    "padDigits",
    "hex_md5",
    "str_md5",
    "wgetText",
    "wgetXML",
    "supplant",
    "bufWriteLine",
    "bufWrite",
    "bufReadAll",
    "bufPeek",
    "copyPlaintextToClipboard",
    "getFormattedCurrency",
    "dumper",
    "getDDMachineList",
    
    /* OBJECTS */
    "Deal",
    "seti",
    "ProductURL",
    
    /* from merchant list code */
    "merchantList",
    "merchantListRids"
]

////////////////////////////////////////////////////////////////////////////////
// GLOBAL CONSTANTS
////////////////////////////////////////////////////////////////////////////////

var EULA_VERSION_LATEST = 0; //update this whenever the EULA is revised
var DDFIREFOX_EXT_ID = "ddfirefox@dynamitedata";

var SERVER_USERNAME = "u00000600";

var MERCHANT_LIST_PREFIX = "http://";
var MERCHANT_LIST_POSTFIX = "/cgi-bin/xml_request.pl?index=reseller&agent=ddFirefox%2F1.4.0"; // + Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager).getItemForID("ddfirefox@dynamitedata").version;

var MERCHANT_IMAGE_URL = "http://www.dynamitedata.com/cgi-bin/display_image.pl?reseller={RID}&file=120x42s.png"
/* use the following for this:
  imageURLReseller = {RID: 'r00000002'};
  finalresult = supplant(MERCHANT_IMAGE_URL, imageURLReseller);
*/

var PRODUCT_INFO_PROTOCOL = "http://";
var PRODUCT_INFO_ARGSPREFIX = "/cgi-bin/search.pl?user=" + SERVER_USERNAME + "&expand=3&format=xml&timeout=5&q=";
var PRODUCT_INFO_ARGSPOSTFIX = "";
var PRODUCT_INFO_MAXRESULTS = 3;
var PRODUCT_INFO_LISTPOSTFIX = "&country=";
var PRODUCT_INFO_REFRESH_TIMEOUT = -1; //60000; //set to -1 to disable

var MESSAGE_HIDE_DELAY_MSECS = 15000;

var LIST_COUNTRIES_NOEXPANSION = new Array("us", "uk", "ca"); //countries which will not pop up the "help us expand" message
var LIST_COUNTRIES_OVERRIDES = new Array(); // countries available for the override feature
LIST_COUNTRIES_OVERRIDES["auto"] = "Automatic";
LIST_COUNTRIES_OVERRIDES["us"] = "United States";
LIST_COUNTRIES_OVERRIDES["uk"] = "United Kingdom";
LIST_COUNTRIES_OVERRIDES["ca"] = "Canada";


var SETI_PREFIX = "http://";
var SETI_LIST_POSTFIX = "/cgi-bin/request.pl";
var SETI_SUBMISSION_POSTFIX = "/cgi-bin/submit.pl";
var SETI_SUBMISSION_FORMAT = "url={URL}\ncache={DATA}";

// all times expressed in msec
//     1000 msec = 1 second
//    60000 msec = 1 minute
//  3600000 msec = 1 hour
// 86400000 msec = 1 day
var STATE_MANAGER_TIMEOUT_STATUS = 36000000;
var STATE_MANAGER_TIMEOUT_TABS = 36000000;
var STATE_MANAGER_TIMEOUT_ERRORS = 360000;
var STATE_MANAGER_SCAN_INTERVAL = 1000;

var STATUS_NBR_MIN = -2;
var STATUS_NBR_MAX = 5;

var prefprefix = "ddfirefox.";

////////////////////////////////////////////////////////////////////////////////
// global variables
////////////////////////////////////////////////////////////////////////////////
var globals = {

    // finds the domain host and one- or two-section TLD -- access via
    //     returnDomainTLD(url);
    // screwing with this will break detection -- this will probably need to be
    //     synchronized with the server
    
    regexFindDomain: /https?:\/\/(?:www\d?\.)?([\w\d\-\.]+\.[\w]{2,3}(?:\.\w{2,3})?)\/?/,
    regexFindURL: /https?:\/\//,

    // interval ID for the seti Scanner
    // note: you can test for firefox started by seeing if this variable is
    //     undefined or has a value
    setiScannerIntervalID: null,

    // prevent installation sequence from running more than once
    updateMessageDisplayAlreadyRunning: false,
    
    // to limit the number of deals the plugin displays to the end user
    // this is also enforced serverside
    maxDealsToDisplay: 3,
    failoverPos: 0,
    
    myIpAddress: "-.-.-.-",

    //currency format to use
    //    currencyFormat : "currencyFormat not set",
    //when toolbar is in release, set this to false
    //   shows/hides beta icon and status controls a few things in the build script
    beta: false,
    prefprefix: "ddfirefox.", //debugging temp
    startupWasRun: false,
    cachePrefKey: "merchantlist.cache",
    version: "",
    
    textErrorAcceptEULA: "You must accept the Dynamite Deals EULA before using this add-on.",
    textErrorRateLimit: "Dynamite Deals is not working because your internet connection exceeded the daily limit for requests. Please try again tomorrow.",
    textErrorLookupFail: "An error occurred while trying to perform a search. Try refreshing the page to attempt your search again. If this happens often, please inform us via the Feedback link below.",
    textErrorXMLFail: "Dynamite Deals was unable to successfully process this page. Try refreshing the page to attempt your search again. If this happens often, please inform us via the Feedback link below.",
    textErrorMerchantDownloadFail: "Dynamite Deals was unable to get a list of merchants to watch for. Please make sure your internet connection is working and restart Firefox.",
    textErrorMerchantDownloadBlocked: "Something is blocking Dynamite Deals from contacting its servers, preventing this add-on from working. Please check your network connection or contact your system administrator.",
        
    textMessageNotOnMerchantPage: "If you would like to suggest this site as a merchant to include in Dynamite Deal's database, let us know! Click the 'Feedback' link below this text.",
    textMessageOnMerchantNotBuyPage: "To find the lowest prices on the web, go to a page on this merchant showing only the product you are looking for.",
    textMessageOnMerchantIsOnBuyPage: "This is a supported merchant, but Dynamite Deals doesn't have any information for this page.",
    textMessageOnRotatingStick: "Dynamite Deals is currently searching its database to find you the best deal. Only a few more seconds...",
    
    textErrorTimeout: "The Dynamite Deals search timed out. You may try your search again by refreshing the page. If this happens often, please inform us via the Feedback link below."
        
}

// stateTable is an array of objects stateStruct
// hashes should be either one of two types of data:
//  * a owning tab ID, for storing deal messages
//      (to be recalled when user tab changes)
//      statusNbr 
//  * a XMLHttpRequest async randomly generated ID
//      for tracking download status
//
// - Individual actors update their status on the state table, 
// - Individual actors are responsible for clearing their statuses,
//      timestamp function will serve as timeout backup
// - The state manager monitors the state table and current tab ID
//   to decide what to draw in the status bar, and runs on an interval
// DO NOT ADD CUSTOM PROPERTIES OR ANY OTHER FUNCTIONS TO stateTable!!!!!
//    doing so could break for/each
var stateTable = new Array();

////////////////////////////////////////////////////////////////////////////////
// Servers and URLs
////////////////////////////////////////////////////////////////////////////////

var SERVER_DEBUG = "ignition.dynamitedata.com";
var SERVER_RELEASE = "detonator.dynamitedata.com";

var failoverChain = new Array();
failoverChain[0] = SERVER_RELEASE;
failoverChain[1] = "detonator.dynamitedata.net";

var urlTable = new Array();

urlTable["main"] = "http://www.dynamitedeals.com";
urlTable["beta"] = "http://toolbar.dynamitedata.com/ddfirefox/notes/";
urlTable["changelog"] = "http://toolbar.dynamitedata.com/ddfirefox/notes/";
urlTable["welcome"] = "http://www.dynamitedeals.com/download/firstrun/";
urlTable["privacy"] = "http://www.dynamitedeals.com/privacy/";
urlTable["commercial services"] = "http://www.dynamitedata.com/solutions/";
urlTable["report a problem"] = "http://www.dynamitedata.com/cgi-bin/contact.pl";
// urlTable["eula"] = "chrome://ddfirefox/content/eula.html";
urlTable["eula"] = "http://www.dynamitedeals.com/download/eula/firefox.html";
urlTable["help"] = "http://www.dynamitedeals.com/download/help/";

function getDDMachineList(failoverPos) {

    if (!failoverPos) failoverPos = globals.failoverPos;
    
    // set server based on failover order
    if (failoverChain[failoverPos]) {
        
        dprint("getDDMachineList: Setting serverlist with failoverPos " + failoverPos);
        ddSetStringPref("merchantListURL", MERCHANT_LIST_PREFIX + failoverChain[failoverPos] + MERCHANT_LIST_POSTFIX);
        ddSetStringPref("productInfoURL", PRODUCT_INFO_PROTOCOL + failoverChain[failoverPos] + PRODUCT_INFO_ARGSPREFIX);
        ddSetStringPref("bgListURL", SETI_PREFIX + failoverChain[failoverPos] + SETI_LIST_POSTFIX);
        ddSetStringPref("bgSubmissionURL", SETI_PREFIX + failoverChain[failoverPos] + SETI_SUBMISSION_POSTFIX);

        dprint("merchantListURL = " + ddGetStringPref("merchantListURL"), 2);
        dprint("productInfoURL = " + ddGetStringPref("productInfoURL"), 2);
        
    } else {
        dprint("getDDMachineList: Not set due to invalid failoverPos");
    }

}



////////////////////////////////////////////////////////////////////////////////
// global objects
////////////////////////////////////////////////////////////////////////////////
function Deal() {
    //prices
    this.price;
    this.rebate;
    this.finalprice;
    this.shipping;
    this.estimate;

    //stock level
    this.stock;

    // link url
    this.url;

    // info on retailer
    this.retailer;
    this.retailerID;
    this.country;
    this.zip;
    
    this.pos;
}

// product URL
// - stores all information related to a given product URL
//   including its dealset, tab IDs, and other goodies related to managing
//   the relationship between deals and what pages the user is browsing
//
// - to use this, instantiate the object first, and then copy a temp dealset
//   object to the instantiated productURL's deals variable'
//
function ProductURL() {
    this.deals = new Array();
    //this.ownerTab = browserTabID;
    this.url;
    this.id;
    this.name;
    this.currencyUnit;

    // flag this deal if the origin site has the best price
    this.originHasBestPrice;
    
    // exclusive?
    this.exclusive; //deprecated
        
    //clear.gif
    this.trackingURL;
    
    //results territory
    this.territory;
    
    //is the user looking at a buy page?
    this.isABuyPage;
    
    //message to display
    this.message;
}

////////////////////////////////////////////////////////////////////////////////
// private variables
////////////////////////////////////////////////////////////////////////////////
// verbosity:
// 0 = none (for release)
// 1 = normal
// 2 = talky
// 3 = even more talky!
//
// To see debug messages:
// 1) launch firefox with -console or from tty console
// 2) make sure browser.dom.window.dump.enabled = true in about:config
var debuglevel = 1;

//WORKAROUND for nsITimer
var localTimerSetiTotalSeconds = 0;

//for the console dump buffer
var textBuffer = "";
var textBufferMaxLength = "50000";

///// examples from https://developer.mozilla.org/en/The_Thread_Manager
// init bg and then main thread
var background = Components.classes["@mozilla.org/thread-manager;1"].getService().newThread(0);
var main = Components.classes["@mozilla.org/thread-manager;1"].getService().mainThread;

const ddFirefoxID = "ddfirefox@dynamitedata";
var ddFirefoxUninstallOnExit;
var appObserver;

var timer;

var setiNeedsToDie; // set to true and SETI will quit ASAP

////////////////////////////////////////////////////////////////////////////////
// global startup & global interfaces
////////////////////////////////////////////////////////////////////////////////
//var startupWasRun = false;
function globalStartup() {

    dprint("Precheck (GLOBAL): startupWasRun == " + globals.startupWasRun);

    if (globals.startupWasRun == false) {
        dprint("Beginning boot-up routine...");

        // set up the background interval for seti
        var interval = ddGetIntPref("bgInterval");
        //   ^^^^^^ moved to local vars section above
        // ** WORKAROUND **
        // timer does not seem to like intervals > 10,000 or (sometimes) 16,384.
        // It won't run at all at intervals higher than those values. WHY?!
        // for now, run timer on 1sec interval with calling function set to workaroundSetiRunner
        // workaroundSetiRunner will increment once per second and execute seti.run
        // when total time elapsed is equivalent to bgInterval pref
        var workaroundSetiRunner = function() {
            localTimerSetiTotalSeconds++;
            //dprint("SETI workaround localtimer = " + localTimerSetiTotalSeconds +
            //       ", interval (secs) = " + Math.round(interval / 1000) + " => " +
            //       (Math.round(interval / 1000) == localTimerSetiTotalSeconds));
            if (Math.round(interval / 1000) == localTimerSetiTotalSeconds) {
                localTimerSetiTotalSeconds = 0;
                var timerWorkAround = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
                timerWorkAround.initWithCallback(seti.run, 10, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
            }
        }

        timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
        timer.initWithCallback(workaroundSetiRunner, 1000, Components.interfaces.nsITimer.TYPE_REPEATING_PRECISE);
        dprint("Background scanner interval set. Timeout = " + interval);

        if (getDebugLevel() > 2) {
            // run seti immediately if we're in high debug mode
            dprint("Running SETI once due to debuglevel > 1...");
            var timerTmp = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
            timerTmp.initWithCallback(seti.run, 1000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
        }

        // listen for uninstall + quit
        appObserver = new globalEventObserver();
        
        // get our current IP address for logging purposes
        //globals.myIpAddress = getMyIPAddress(); //moved to merchants xml
        
        // ok we're done
        globals.startupWasRun = true;
        dprint("Boot-up routine completed. startupWasRun is now " + globals.startupWasRun);

    }

}

////////////////////////////////////////////////////////////////////////////////
// currency and localization
////////////////////////////////////////////////////////////////////////////////
function getFormattedCurrency(amt, ISO) {
    dprint("Formatting " + amt + " for " + ISO, 3);

    var curType = new Array();

    // make sure we have dollars and cents
    amt = Number(amt).toFixed(2).toString();

    ISO = ISO.toUpperCase();

    //amt = amt.toFixed(2);
    var amtTmp = amt.toString().split(".");
    var amtDOL = amtTmp[0];
    var amtCEN = amtTmp[1];

    var toReturn;

    //// CURRENCY FORMAT DECLARATIONS
    //format: "currency display|thousands seperator"
    // {DOL} = large denomination
    // {CEN} = small denomination
    curType['USD'] = "${DOL}.{CEN}|,";
    curType['CAD'] = "${DOL}.{CEN}|,";
    curType['EUR'] = "{DOL},{CEN} \u20AC|.";
    curType['GBP'] = "\u00A3{DOL}.{CEN}|,";
    curType['JPY'] = "\u00A5{DOL}|,";
    curType['CNY'] = "\u00A5{DOL}.{CEN}|,";
    curType['KRW'] = "\u20A0{DOL}|";
    curType['NOK'] = "{DOL} kr|";
    curType['SEK'] = "{DOL} kr|";
    curType['HKD'] = "${DOL}.{CEN}|,";
    curType['TWD'] = "${DOL}.{CEN}|,";

    if (curType[ISO]) { //currency type known
        // parse formatting instructions
        var splitTmp = curType[ISO].split("|");
        var amtFormat = splitTmp[0];
        var seperator = splitTmp[1];

        var sepLen = 3;
        var amtDOLnew;

        function checkSpacer(a) {
            dprint("amtDOLnew(a) is " + a, 3);
            if (a) {
                return a;
            } else {
                return "";
            }
        }

        //insert the unit seperator if necessary
        if (seperator != "") {
            var x;

            for (x = amtDOL.length; x > sepLen; x = x - sepLen) {
                amtDOLnew = seperator + amtDOL.slice(Math.max(x - sepLen, 0), x) + checkSpacer(amtDOLnew);
            }

            // grab the remaining digits
            amtDOLnew = amtDOL.slice(0, x) + checkSpacer(amtDOLnew);

            amtDOL = amtDOLnew;
        }

        //return formatted currency unit
        return supplant(amtFormat, {
            DOL: amtDOL,
            CEN: amtCEN
        })

    } else { // unknown currency type
        return ISO + " " + amt;
    }
}

////////////////////////////////////////////////////////////////////////////////
// stuff for the state manager
////////////////////////////////////////////////////////////////////////////////
//states are stored as hash table of objects stateStruct, keyed by state ID
// stateID is a random number for background/get/etc events
// stateID is a pointer to a browser tab for deal notifications
// using javascript's ad-hoc hash support
// http://www.mojavelinux.com/articles/javascript_hashes.html
function stateStruct() {
    this.owner;
    //owning tab, if applicable (though this should also be in the hash)
    this.statusText; // status text;
    this.statusURL;

    this.statusNbr; // status codes, defined as follows:
    /// error and message states:
    //      store error/message text in statusTextl
    //      store error data/object/whatever in statusData
    //      try to keep the timeouts on these short, so that we don't
    //          obstruct the statusbar for too long
    // -2 = message (statusText = message, statusURL = URL)
    // -1 = error of some sort
    
    /// normal states
    // 0 = nothing
    // 1 = foreground downloading (getting merch list, prod info, etc)
    // 2 = background operations (for the background SETI scanner)
    // 3 = deal found!
    //     (statusText will contain the text to be displayed)
    // 4 = window (not sure if we're going to use this)
    // 5 = cookie killer
    // 6 = onPageLoad() searching
    // 7 = merchant known but no results
    this.statusData;

    //this.timestamp;
    this.timeout;

}

var stateManager = {

    // adds a new state to the stateTable, or edits an existing ID
    // set ID to either null or undefined to add an entry
    // specify an existing ID to edit one
    // set failIfIDDoesntExist to true to force addEditState to abort if
    //   requested ID doesn't exist (was deleted, timed out, etc)
    addEditState: function(ID, statusNbr, statusText, statusURL, timeout, statusData, failIfIDDoesntExist) {        

        var date = new Date(); // for time stuff
        // check to see if ID exists... fail if needed
        if (!stateTable[ID]) {
            if (failIfIDDoesntExist) {
                return - 2;
            } else if ((ID === null) | (ID === undefined)) {
                ID = stateManager.generateRandomID();
                dprint("stateManager: addEdit created ID" + ID)
            }
        } else {
        	// This is a workaround for a multiple pending lookups on the same tab.
        	if ( (statusNbr == 3 || statusNbr == 7) && stateTable[ID].statusURL != statusURL ) {
        		dprint("stateManager: addEdit failed because query is stale.");
        		return -1;
        	}
        }

        if (!statusNbr) {
            dprint("stateManager: addEdit failed because no statusNbr was specified!");
            return -1;
        }

        /*if ((statusNbr >= STATUS_NBR_MIN) & (statusNbr <= STATUS_NBR_MAX)) {
            // good to go
        } else {
            dprint("stateManager: addEdit failed because statusNbr was outside the acceptible range!");
            dprint("^^^^ " + STATUS_NBR_MIN + " <= " + statusNbr + " <= " + STATUS_NBR_MAX + " != true");
            return -1;
        }*/
        
        var tmpState = new stateStruct();

        tmpState.statusNbr = statusNbr;

        if (typeof(statusText) != 'undefined') {
            tmpState.statusText = statusText;
        }
        if (typeof(statusURL) != 'undefined') tmpState.statusURL = statusURL;
        if (typeof(statusData) != 'undefined') tmpState.statusData = statusData;
        if (typeof(timeout) != 'undefined') {
            tmpState.timeout = date.getTime() + timeout * 1000;
        } else {
            switch (statusNbr) {
            case 3:
                {
                    tmpState.timeout = date.getTime() + STATE_MANAGER_TIMEOUT_TABS;
                    break;
                }
            case - 1 : {
                    tmpState.timeout = date.getTime() + STATE_MANAGER_TIMEOUT_ERRORS;
                    break;
                }
            default:
                {
                    tmpState.timeout = date.getTime() + STATE_MANAGER_TIMEOUT_STATUS;
                }
            }
        }

        dprint("state manager: ADDED ID " + ID, 2);
        stateTable[ID] = tmpState;

        if (getDebugLevel > 1) stateManager.dumpStateTable();

        return ID;

    },

    // delete entry from state table by ID
    deleteState: function(ID) {
        //stateTable.splice(ID, 1); // doesn't seem to work for hashtables
        if (ID) {
            //stateTable[ID] = undefined;
            delete(stateTable[ID]);
            dprint("state manager: DELETED ID " + ID, 2);
        } else {
            dprint("state manager: ignoring request to delete undefined state", 2);
        }

        /////// DEBUG CODE //////////
        if (getDebugLevel() > 1) stateManager.dumpStateTable();
    },

    generateRandomID: function() {

        // generate a sorta GUID thing
        function S4() {
            return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
        }

        return (S4() + "-" + S4()
        /*S4()+"-"+S4()+S4()*/
        ).toUpperCase();
        
    },

    dumpStateTable: function() {
        dprint("==== State table contains: ===", 2);

        for (i in stateTable) {
            if (stateTable[i]) dprint(i + ", " + stateTable[i].statusNbr, 2);
        }
        dprint("==============================", 2);
    }
}

////////////////////////////////////////////////////////////////////////////////
// debug stuff
////////////////////////////////////////////////////////////////////////////////
function dprint(message, level) { //print debug info
    ddprefix = "ddF";
    cr = "\n";

    if (level == undefined) {
        level = 1; // default print to debug level 1
    }

    if (debuglevel >= level) {
        
        // form message
        
        var time = new Date();
        var newmessage = "[" +
            padDigits(time.getUTCFullYear(), 4) +
            padDigits(time.getUTCMonth() + 1, 2) +
            padDigits(time.getUTCDate(), 2) + "-" +
            padDigits(time.getUTCHours(), 2) +
            padDigits(time.getUTCMinutes(), 2) +
            padDigits(time.getUTCSeconds(), 2)/* +
            padDigits(time.getUTCMilliseconds(), 4)*/ + " " +
            globals.myIpAddress + " " + ddprefix + "] " + message + cr;
        
        // debug printing code goes here
        dump(newmessage);
        bufWrite(newmessage);
    }
}

function getDebugLevel() {
    return debuglevel;
}

function setDebugLevel(num) {
    debuglevel = num;
    dprint("!!! Debug level set to " + num + ".", 0);
}

// use this because FF is complaining that isNaN() "is not defined"
function isNumeric(sText) {
    //var ValidChars = "0123456789.";
    //var IsNumber = true;
    //var Char;
    //
    //for (i = 0; ((i < sText.length) & (IsNumber == true)); i++) {
    //    Char = sText.charAt(i);
    //    if (ValidChars.indexOf(Char) == -1) {
    //        IsNumber = false;
    //    }
    //}
    //
    //return IsNumber;
    return ! isNaN(sText);

}

// ugh... this doesnt work as it should...
function isAlive(s) {
    if (s.IamOnlyCheckignForNullOrUndefined) {
        return true;
    } else {
        return false;
    }
}

// clones an object to get around JS' pass-by-reference habit
// returns a cloned copy of the object supplied in param what
function CloneObject(what) {
    for (i in what) {
        this[i] = what[i];
    }
}

// pads a number with leading zeroes
// http://classicasp.aspfaq.com/general/how-do-i-pad-digits-with-leading-zeros.html
function padDigits(n, totalDigits) {
    n = n.toString();
    var pd = '';
    if (totalDigits > n.length) {
        for (i = 0; i < (totalDigits - n.length); i++) {
            pd += '0';
        }
    }
    return pd + n.toString();
}

////////////////////////////////////////////////////////////////////////////////
// preferences
////////////////////////////////////////////////////////////////////////////////
var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

function ddGetStringPref(prefname) {
    var p = prefs.getCharPref(prefprefix + prefname);
    dprint("got string pref " + prefname, 2);
    dprint("^^^^ " + p, 2);
    return p;
}

function ddSetStringPref(prefname, value) {
    dprint("setting string pref " + prefname, 2);
    dprint("^^^^ " + value, 2);
    prefs.setCharPref(prefprefix + prefname, value);

}

function ddGetIntPref(prefname) {
    var p = prefs.getIntPref(prefprefix + prefname);
    dprint("got int pref " + prefname, 2);
    dprint("^^^^ " + p, 2);
    return p;
}

function ddSetIntPref(prefname, value) {
    dprint("setting int pref " + prefname, 2);
    dprint("^^^^ " + value, 2);
    prefs.setIntPref(prefprefix + prefname, value);
}

function ddGetBoolPref(prefname) {
    var p = prefs.getBoolPref(prefprefix + prefname);
    dprint("got Bool pref " + prefname, 2);
    dprint("^^^^ " + p, 2);
    return p;
}

function ddSetBoolPref(prefname, value) {
    dprint("setting bool pref " + prefname, 2);
    dprint("^^^^ " + value, 2);
    prefs.setBoolPref(prefprefix + prefname, value);
}

////////////////////////////////////////////////////////////////////////////////
// WWW I/O
////////////////////////////////////////////////////////////////////////////////
// returns XMLHTTPRequest object if success
// returns HTTP status code 
// https://developer.mozilla.org/en/XMLHttpRequest
// do not use this outside this js file
// * generally you want to send use async = true
//   - however async = false works well in event handlers
// * assumes false is async is not specified *
// * Note that wgetXML and wgetText function identically when async == true
//   - passes req object so caller can set event handlers <kludge/>
function wgetXML(url, async, mime, killCookies) {
        
    dprint("wgetXML() called", 2);
    dprint("^^^^ " + url, 2);

    req = wgetInternal(url, async, mime, killCookies);

    if (async == true) {
        return req;
    } else {
        if (req.status == 200) { // HTTP 200 OK
            dprint("WgetXML passed", 3);
            return req.responseXML;
        } else {
            dprint("WgetXML failed, status code " + req, 2);
            return req; // return the error code, given from wgetInternal()
        }
    }

}

function wgetText(url, async, mime, killCookies) {
    dprint("wgetText() called", 2);

    req = wgetInternal(url, async, mime, killCookies);

    if (async == true) {
        return req;
    } else {
        if (req.status == 200) { // HTTP 200 OK
            return req.responseText;
        } else {
            dprint("wgetText returning " + req + "...", 2)
            return req; // return the error code, given from wgetInternal()
        }
    }

}

function wgetInternal(url, async, mime, killcookies) {
    var returnData;

    var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();
    if (mime) req.overrideMimeType = mime;
    
    
    if (async == true) {
        /* the following code segment was written but has never been used or
          tested. If we want to use since the sync requests have been updated
          several times this section probably needs to be rewritten before
          being used. */
        
        // add stateTable entry
        state = stateManager.addEditState("wgetA|" + stateManager.generateRandomID(), 1);

        //process XMLHTTPRequest asynchroniously
        req.open('GET', url, true);

        try {
            
            req.DDstate = state;
            
            if (killcookies) {
                
                dprint("wgetInternal() " + state + " killing cookies (async)...")
            
                var ck = new CookieKillah(req, 500);
                
                req.onreadystatechange = function(i) {
                    if (req.readyState == 4) {
                        dprint("wgetInternal() " + state + " completed...", 2)
                        stateManager.deleteState(state);
                    }
                }
                
                
                req.send(null); // sends request, null = no body
                //cookieKillah.abort();
                
            } else {
                
                req.send(null);
                    
            }
            
        } catch(e) {
            
            //if (cookieKillah) cookieKillah.abort();
            dprint("wgetInternal() (async)" + state + " crashed: " + ex);
            return -1;
        }
        // clear stateTable entry when download finishes
        // note that caller needs to get data in its own event handler

        // pass XMLHTTPRequest handle to caller,
        // caller will need to create its on event handler to get the data
        // that downloaded.
        return req;
        
    } else {
        // process XMLHTTPRequest synchroniously
        var failoverRetry = true;
        while (failoverRetry == true) {
            
            failoverRetry = false;
            
            req.open('GET', url, false); // prepare url request, set as sync
            
            // add stateTable entry
            state = stateManager.addEditState("wget|" + stateManager.generateRandomID(), 1);
            
            dprint("wgetInternal() " + state + " forming request...");
            dprint("wgetInternal() " + state + " " + url, 2)
            // send the request
            try {
                
                if (killcookies) {
                    
                    dprint("wgetInternal() " + state + " killing cookies...")
                
                    var ck = new CookieKillah(req, 500);
                    
                    req.send(null); // sends request, null = no body
                    //cookieKillah.abort();
                    
                } else {
                    
                    req.send(null);
                        
                }
                
            } catch(e) {
                
                //if (cookieKillah) cookieKillah.abort();
                dprint("wgetInternal() " + state + " crashed: " + ex);
                return -1;
            }
            
            // evaluate the response, if 200 then we're good
            // this likes to crash so we wrap it in try{}
            try {
                
                if ((req.status == 200) | (req.status == 0)) { // HTTP 200 OK / local file ok
                    
                    dprint("wgetInternal() " + state + " Download successful, retrieved " + req.responseText.length + " characters.");
                    dprint("^^^^ " + url, 2)
                    //dumpResponseToTempFile(req.responseText);
                    returnData = req;
                
                } else if (req.status == 403) { //blocked, roll over next failover
                        
                        dprint("wgetInternal() " + state + " Request appears to have been blocked. (" + req.status + ")...");
                        returnData = req.status;
                    
                } else {
                
                    dprint("wgetInternal()  " + state + " Download failed. returned " + req.status);
                    returnData = req.status;
                }
                
            } catch(ex) {
                
                dprint("wgetInternal()  " + state + " crashed: " + ex);
                //returnData = "";
                
            }
    
            // clear state entry
            stateManager.deleteState(state);
    
            // no, Mr. Bond. I expect you to die.
            
        }
        
        return returnData;
        

    }

}

//////////////////////////////////////////////////////////////////////////////////
//// cookie killer
//////////////////////////////////////////////////////////////////////////////////
function CookieKillah (xmlreq, timeout) {
    
    //add state that we can monitor
    this.stateID = stateManager.addEditState("ck|" + stateManager.generateRandomID(), 5)
    dprint(this.stateID + " Instantiated...", 2);
    
    if (!timeout) timeout = 1000; //msec
    
    //init stuff
    this.mySubject = xmlreq.channel;    
    this.myTopic = "http-on-modify-request";
    
    // create a new observer
    this.obs = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
    this.obs.addObserver(this, this.myTopic, false);
    
    //kill this object after timeout
    this.timerTmp = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
    this.timerTmp.initWithCallback(this.abort, timeout, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
    
    dprint(this.stateID + " Ready!", 2);
    
}

CookieKillah.prototype.observe = function(subject, topic, data) {
    
    dprint(this.stateID + " Intercepting cookie...", 2);
    
    // check to see if this xmlhttprequest is ours or not
    if (topic == this.myTopic & subject == this.mySubject) {
        this.mySubject.QueryInterface(Components.interfaces.nsIHttpChannel);
        this.mySubject.setRequestHeader("Cookie", "", false);
        dprint(this.stateID + " Killed!", 2);
        this.timerTmp.cancel(); //stop the timeout self-destruct
        this.abort(); //doesnt work right... caller has responsibility to abort
    } else {
        dprint(this.stateID + " Ignored!", 2);
        return;
    }
    
}

CookieKillah.prototype.QueryInterface = function(x) {
    if (x.equals(Components.interfaces.nsISupports) || x.equals(Components.interfaces.nsIObserver))
        return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
}

CookieKillah.prototype.abort = function()  {
    
    this.obs.removeObserver(this, this.myTopic);
    
    delete(this.obs);
    delete(this.mySubject);
    delete(this.myTopic);
    
    dprint(this.stateID + " Aborting", 2);
    stateManager.deleteState(this.stateID);
    
    delete(this.stateID);
}

////////////////////////////////////////////////////////////////////////////////
// background updater
////////////////////////////////////////////////////////////////////////////////
//var urlQueue = new Array();

var seti = {

    // gets the URL list from server, parses it, and returns an array
    // list containing the URLs to scan
    getURLList: function() {

        url = ddGetStringPref("bgListURL");
        dprint("SETI: Getting list of URLs to scan...");
        dprint("SETI: getting list from " + url, 2);

        urlQueueRaw = wgetText(url, false);
        return urlQueueRaw;
    },

    getURL: function(url) {
        var res = wgetText(url, false, 'text/plain', true);
        return res;
    },

    parseList: function(list) {

        return list.split(/\r\n|\r|\n/);
    },

    submitResults: function(url, results) {
        // get submission URL
        var submissionURL = ddGetStringPref("bgSubmissionURL");

        dprint("SETI: ^^^^ sending...." + url);

        // prepare results for transmission
        var feedToSupplant = {
            URL: url,
            DATA: results
        };
        //var postBack = supplant(SETI_SUBMISSION_FORMAT, feedToSupplant);
        var postBack = "url=" + encodeURIComponent(url) + "&cache=" + encodeURIComponent(results);

        // transmit
        var req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance();
        req.open('POST', submissionURL, false);
        req.setRequestHeader("Content-type", "application/x-www-form-urlencoded")
        //req.overrideMimeType = 'text/plain';
        req.send(postBack);

        // error checking, we want HTTP 200 OK
        if ((req.status == 200)) {
            dprint("SETI: ^^^^ sent. Server replied: " + req.responseText);
            return 0;
        } else {
            dprint("SETI: ^^^^ FAILED! status = " + req.status);
            return req.status;
        }

    },

    run: function() {
        function runInThread() {
            // run threaded version of seti
            // bg and main thread constructors moved to local globals
            background.dispatch(new workingThread(1, main, stateID), background.DISPATCH_NORMAL);
        }

        try {
            if (!ddGetBoolPref("trueMultithreading")) {

                // the OLD style
                stateID = stateManager.addEditState("SETI_SCANNER", 2);
                dprint("SETI: Multithreading support is currently disabled...");
                seti.runInternal();
                stateManager.deleteState(stateID);

            } else {

                // the NEW style
                stateID = stateManager.addEditState("SETI_SCANNER", 2);
                dprint("SETI: Multithreading support is currently enabled...");
                runInThread();

            }
        } catch(e) {
            dprint("SETI: Crashed while accessing multithread setting, MT disabled!");
            seti.runInternal();
        }
    },

    runInternal: function(override) {
        var stateID;

        // check to see if user is allowing background scan
        if (ddGetBoolPref("allowBackgroundScanning") || override) {

            try {

                dprint("SETI: Background scan started...");

                var req = seti.getURLList();
                req.replace(/(\n|\r)+$/, '');
                var results;
                var i;

                dprint("^^^^ Does the download appear to be a list of URLs? " + !isNumeric(req));	
                dprint(req);

                try {
                    var urlQueue = req.split(/\r\n|\r|\n/);
                    dprint("SETI: urlQueue has " + urlQueue.length + " entries.");

                    // we have a list of URLs, lets get to work
                    if ((!isNumeric(req)) & (urlQueue.length > 0)) {
                        // we have a list of URLs, lets get to work
                        dprint("^^^^ check passed. Beginning operation...")

                        //for each (var i in urlQueue) {
                        for (i = 0; i < (urlQueue.length); i++) {
                            dprint("SETI: ==== GET #" + i + "====")

                            if (setiNeedsToDie) {
                                return;
                            }

                            //check to make sure this entry is a URL
                            var isURL = globals.regexFindURL.exec(urlQueue[i]);

                            if (isURL && isURL[0]) {

                                try {

                                    dprint("SETI: getting.... (" + urlQueue[i] + ")");
                                    results = seti.getURL(urlQueue[i]);

                                    if (!isNumeric(results)) {

                                        dprint("SETI: GET #" + i + ": retreived! Submitting...");
                                        seti.submitResults(urlQueue[i], results);

                                    } else {

                                        dprint("SETI: GET #" + i + ": failed. HTTP status " + results);

                                    }

                                } catch(ex) {

                                    dprint("SETI: GET #" + i + ": CRASHED!");
                                    dprint("^^^^ Error: " + ex);
                                }

                            } else {

                                dprint("SETI: List entry #" + i + " does not appear to be a URL. Skipping...");

                            }

                        }

                    } else {
                        dprint("SETI: Could not get list of URLs to scan. Status code " + req);
                        dprint("SETI: Stopping, as I have nothing to do.");
                    }
                } catch(ex) {
                    dprint("SETI: CRASHED on parse!");
                    dprint("^^^^ Error text:" + ex);
                }

                stateManager.deleteState(stateID);
                return true; // true means that it tried to run, regardless of whether or not it failed
            } catch(ex) {
                dprint("SETI: CRASHED after start but before parse!");
                dprint("^^^^ Error text:" + ex);
                return false;
            }

        } else {
            dprint("SETI: Skipping due to user preference...");
            return false;
        }

    }

}

////////////////////////////////////////////////////////////////////////////////
// dump buffer stuff
////////////////////////////////////////////////////////////////////////////////
// is there some kind of text stream we could use?
function bufWrite(msg) {
    textBuffer = textBuffer + msg;
    if (textBuffer.length > textBufferMaxLength) {
        textBuffer = textBuffer.slice(textBuffer.length - textBufferMaxLength)
    }
}

function bufWriteLine(msg) {
    bufWrite(msg + "\n")
}

function bufReadAll() {
    var tmp = textBuffer;
    textBuffer = "";
    return tmp;
}

function bufPeek(chars) {
    
    var tmp;
    
    if (!chars) {
        var tmp = textBuffer;    
    } else {
        var len = textBuffer.length - 1;
        var tmp = textBuffer.substring(len - chars, len);
    }
    
     return tmp;
}

////////////////////////////////////////////////////////////////////////////////
// multithreading support
// https://developer.mozilla.org/en/The_Thread_Manager
////////////////////////////////////////////////////////////////////////////////
var workingThread = function(threadID, something, statecode) {
    this.threadID = threadID;
    this.result = 0;
    this.stateID = statecode;
};

workingThread.prototype = {
    run: function() {
        try {

            seti.runInternal();
            // call the mainthread to let us know its finished
            main.dispatch(new mainThread(this.threadID, this.result, this.stateID), background.DISPATCH_NORMAL);

        } catch(err) {
            Components.utils.reportError(err);

            this.result = -1;

            // call the mainthread to let us know its finished
            main.dispatch(new mainThread(this.threadID, this.result, this.stateID), background.DISPATCH_NORMAL);
        }
    },

    QueryInterface: function(iid) {
        if (iid.equals(Components.interfaces.nsIRunnable) || iid.equals(Components.interfaces.nsISupports)) {
            return this;
        }
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
};

var mainThread = function(threadID, result, statecode) {
    this.threadID = threadID;
    this.result = result;
    this.stateID = statecode;
};

mainThread.prototype = {
    run: function() {
        try {
            // This is where we react to the completion of the working thread.
            dprint("SETI: Thread ended with result code " + this.result);
            stateManager.deleteState(this.stateID);
        } catch(err) {
            Components.utils.reportError(err);
            stateManager.deleteState(this.stateID);
        }
    },

    QueryInterface: function(iid) {

        if (iid.equals(Components.interfaces.nsIRunnable) || iid.equals(Components.interfaces.nsISupports)) {
            return this;
        }

        throw Components.results.NS_ERROR_NO_INTERFACE;

    }
};

////////////////////////////////////////////////////////////////////////////////
// misc support functions
// most of this is copied and/or modified from samples on mozdev
////////////////////////////////////////////////////////////////////////////////
function supplant(orig, o) {
    return orig.replace(/{([^{}]*)}/g,
    function(a, b) {
        var r = o[b];
        return typeof r === 'string' || typeof r === 'number' ? r: a;
    });
};

function getMyIPAddress() {
    
    //return "XX.XX.XX.XX";
    
    var ipAddressServer = "http://ignition.dynamitedata.com/cgi-bin/get_ip.pl";
    dprint("getMyIPAddress: Getting current IP address...");
    var ipregex = /\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b/;

    var response = wgetText(ipAddressServer, false, "", true);

    var match = ipregex.exec(response);

    if (match != null) {
        result = match[0];
    } else {
    	result = "XX.XX.XX.XX";
    }

    dprint("getMyIPAddress: Current IP appears to be " + result);

    return result;
    
}

/// https://developer.mozilla.org/En/Using_the_Clipboard
function copyPlaintextToClipboard(msg) {

    // prepare data
    var str = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);
    str.data = msg;

    // prepare transfer object
    var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    trans.addDataFlavor("text/unicode");
    trans.setTransferData("text/unicode", str, msg.length * 2);

    // prepare clipboard object
    var clipid = Components.interfaces.nsIClipboard;
    var clip = Components.classes["@mozilla.org/widget/clipboard;1"].getService(clipid);
    clip.setData(trans, null, clipid.kGlobalClipboard);

    if (clip) {
        return true;
    } else {
        return false;
    }

}

/**
* Function : dump()
* Arguments: The data - array,hash(associative array),object
*    The level - OPTIONAL
* Returns  : The textual representation of the array.
* This function was inspired by the print_r function of PHP.
* This will accept some data as the argument and return a
* text that will be a more readable version of the
* array/hash/object that is given.
*/
function dumper(arr, level) {
    var dumped_text = "";
    if (!level) level = 0;

    //The padding given at the beginning of the line.
    var level_padding = "";
    for (var j = 0; j < level + 1; j++) level_padding += "    ";

    if (typeof(arr) == 'object') { //Array/Hashes/Objects
        for (var item in arr) {
            var value = arr[item];

            if (typeof(value) == 'object') { //If it is an array,
                dumped_text += level_padding + "'" + item + "' ...\n";
                dumped_text += dumper(value, level + 1);
            } else {
                dumped_text += level_padding + "'" + item + "' => \"" + value + "\"\n";
            }
        }
    } else { //Stings/Chars/Numbers etc.
        dumped_text = "===>" + arr + "<===(" + typeof(arr) + ")";
    }
    return dumped_text;
}

////////////////////////////////////////////////////////////////////////////////
// extension uninstallation cleanup and quit notification
////////////////////////////////////////////////////////////////////////////////
const OBS_QUIT_REQUEST = "quit-application-requested";
const OBS_QUIT_GRANTED = "quit-application-granted";
const OBS_ACTION_REQUESTED = "em-action-requested";

function globalEventObserver() {
    this.register()
}

globalEventObserver.prototype = {

    observe: function(subject, topic, data) {

        dprint("globalEventObserver: Observed something (T=" + topic + ", S=" + subject.id + ", D=" + data + ")");

        if (topic == OBS_ACTION_REQUESTED) {

            subject.QueryInterface(Components.interfaces.nsIUpdateItem);

            if (subject.id == ddFirefoxID) {

                if (data == "item-uninstalled") {

                    // user is uninstalling the plugin :'-(
                    ddFirefoxUninstallOnExit = true;
                    dprint("globalEventObserver: User wants to uninstall me...");

                } else if (data == "item-cancel-action") {

                    // or not :-D
                    ddFirefoxUninstallOnExit = false;
                    dprint("globalEventObserver: User cancelled uninstall request.");

                }

            }
        } else if (topic == OBS_QUIT_REQUEST) {

            dprint("globalEventObserver: got program exit request! (u = " + ddFirefoxUninstallOnExit);

            setiNeedsToDie = true;
            
            if (ddFirefoxUninstallOnExit == true) {
                
                var prefsO = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    
                dprint("Nuking preferences...");
                var newpre = globals.prefprefix.substring(0, globals.prefprefix.length - 1);
    
                prefsO.deleteBranch(newpre, "");
                dprint("Removed all preferences under " + newpre);
                
            }


        } else if (topic == OBS_QUIT_GRANTED) {

        }
    },

    register: function() {
        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observerService.addObserver(this, OBS_ACTION_REQUESTED, false);
        observerService.addObserver(this, OBS_QUIT_REQUEST, false);
        observerService.addObserver(this, OBS_QUIT_GRANTED, false);
    },

    unregister: function() {
        var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        observerService.removeObserver(this, OBS_ACTION_REQUESTED);
        observerService.removeObserver(this, OBS_QUIT_REQUEST);
        observerService.removeObserver(this, OBS_QUIT_GRANTED);
    }

}

////////////////////////////////////////////////////////////////////////////////
// calculate MD5 hashes
////////////////////////////////////////////////////////////////////////////////

function hex_md5(data) {
    
    // get our byte converter
    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    var bytes = {};
    converter.charset = "UTF-8";
    
    var dataByteArray = converter.convertToByteArray(data, bytes);
    
    // use the hasher
    var hasher = Components.classes["@mozilla.org/security/hash;1"].createInstance(Components.interfaces.nsICryptoHash);
    hasher.init(hasher.MD5);
    hasher.update(dataByteArray, dataByteArray.length);
    var result = hasher.finish(false);
    
    // convert result to string
    function bytesToHexString(byt) {
        var output = "";
        var temp;
        
        for (i in byt) {
            temp = byt.charCodeAt(i);
            // padding with zeroes
            output += String("0" + temp.toString(16)).slice(-2);
        }
        
        return output;
    }
    
    return bytesToHexString(result);
    
}

////////////////////////////////////////////////////////////////////////////////
// merchant list retreival
////////////////////////////////////////////////////////////////////////////////

//fetches the list of merchants in text format

var merchantList = new Array();
var merchantListRids = new Array();